pub enum WaitCondition {
UrlChanged,
A11yContainsText {
text: String,
},
ElementWithName {
name_contains: String,
role: Option<String>,
},
PageLoaded,
}Expand description
Conditions to wait for.
Variants
UrlChanged
Wait for the URL to change.
A11yContainsText
Wait for specific text to appear in the accessibility tree.
ElementWithName
Wait for a specific element to exist in the accessibility tree.
PageLoaded
Wait for page load to complete.