UI state representation for perception pipeline.
UiMap is the core data structure that represents what’s visible on screen in a structured, auditable format.
Structs
- Page
Signals - Text
Block - UiElement
- A single interactable UI element.
- UiMap
- Complete UI state representation.
- UiState
- Element state flags.
Enums
- Element
Source - Icon
Type - Text
Source - UiRole
- Element roles (simplified from full ARIA).