Module models

Module models 

Core browser domain types.

These types represent the perception boundary: what the AI can see and how it can interact with web content.

Structs

A11yNode
An element in the accessibility tree.
Bounds
Rectangle bounds in viewport coordinates.
CookieParam
A cookie to inject into the browser before navigation.
Viewport
Viewport dimensions.

Enums

Modifier
Keyboard modifier keys.
WaitCondition
Conditions to wait for.